/* gendict.c v1.0 -- by Sean Trifero <sean@innu.org> -- solar@IRC
 */ 

#include <stdio.h>
#include <strings.h>
#include <stdlib.h>

char    pctab[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuwvxuz0123456789+-";

#define CIPHER(a1,a2,a3,a4,b1,b2,b3,b4) \
{                                       \
        a2 =  a2 ^  a1;                 \
        b2 =  b2 ^  b1;                 \
        a3 =  a2 ^  a1;                 \
        b3 =  b2 ^  b1;                 \
        b3 >>= 2;                       \
        b3 |= ((a3 & 3) << 30);         \
        a3 >>= 2;                       \
        a2 =  a3 ^  a2;                 \
        b2 =  b3 ^  b2;                 \
        a4 = ~a4 ^  a2;                 \
        b4 = -b4 ^  b2;                 \
        a2 =  a4 ^ ~a2;                 \
        b2 =  b4 ^ -b2;                 \
        b1 >>= 1;                       \
        b1 |= ((a1 & 1) << 31);         \
        a1 >>= 1;                       \
}

#define PASSLEN                         20
#define PASSBUF                         PASSLEN+1

int main(int argc, char *argv[]) {
	char buff[128];
	char *pass;
	char passwd[PASSBUF];
	FILE *input, *output;
		
	if (argc < 3) {
		printf("usage: %s <dict> <hash>\n",argv[0]);
		return 1;
	}

	input = fopen(argv[1], "r");
	output = fopen(argv[2], "w");

	while(!feof(input)) {
		fscanf(input, "%s", buff);
		makepass(passwd,buff);
		fprintf(output, "%s %s\n", passwd, buff);
    	}

        fclose(input);
        fclose(output);
  	printf("done\n");
        return 0;
}

char *cipher(char *arg)
{
        static  char res[40];
        unsigned long   B1a,B2a,B3a,B4a;
        unsigned long   B1b,B2b,B3b,B4b;
        unsigned char   *ptr;
        unsigned long   R1;
        int     i;

        if (!arg || !*arg)
                return(NULL);

        B1a = B2a = B3a = B4a = 0;
        B1b = B2b = B3b = B4b = 0;
        ptr = arg;

        while(*ptr)
        {
                R1 = *ptr;
                for(i=0;i<8;i++)
                {
                        if (R1 & 1)
                        {
                                B1a |= 0x80008000;
                                B1b |= 0x80008000;
                        }
                        R1  >>= 1;
                        CIPHER(B1a,B2a,B3a,B4a,B1b,B2b,B3b,B4b);
                }
                ptr++;
        }
        while((B1a) || (B1b))
        {
                CIPHER(B1a,B2a,B3a,B4a,B1b,B2b,B3b,B4b);
        }

        memset(res,0,sizeof(res));
        i = 0;
        for(i=0;i<10;i++)
        {
                res[i] = pctab[(B4b & 0x3f)];
                B4b >>= 6;
                B4b |= ((B4a & 0x3f) << 26);
                B4a >>= 6;
        }
        return(res);
}

int makepass(char *encoded, char *plain)
{
        strcpy(encoded,cipher(plain));
}
